@@ -5,7 +5,7 @@ import mock
 from nose.tools import eq_
 from nose.tools import raises
 
-from kazoo.client import Callback
+from kazoo.protocol.states import Callback
 
 
 class TestThreadingHandler(unittest.TestCase):
@@ -17,43 +17,6 @@ class TestThreadingHandler(unittest.TestCase):
         from kazoo.handlers.threading import AsyncResult
         return AsyncResult
 
-    def test_completion_vs_session(self):
-        h = self._makeOne()
-        h.start()
-
-        lst = []
-        av = threading.Event()
-        bv = threading.Event()
-        cv = threading.Event()
-        dv = threading.Event()
-
-        def addToList():
-            av.set()
-            bv.wait()
-            lst.append(True)
-            dv.set()
-
-        def anotherAdd():
-            lst.append(True)
-            cv.set()
-
-        call1 = Callback('session', addToList, ())
-        call2 = Callback('completion', anotherAdd, ())
-
-        h.dispatch_callback(call1)
-        av.wait()
-        print
-        # Now we know the first is waiting, make sure
-        # the second executes while the first has blocked
-        # its thread
-        h.dispatch_callback(call2)
-        cv.wait()
-
-        eq_(lst, [True])
-        bv.set()
-        dv.wait()
-        eq_(lst, [True, True])
-
     def test_proper_threading(self):
         h = self._makeOne()
         h.start()
@@ -73,6 +36,15 @@ class TestThreadingHandler(unittest.TestCase):
             raise h.timeout_exception("This is a timeout")
         testit()
 
+    def test_double_start_stop(self):
+        h = self._makeOne()
+        h.start()
+        self.assertTrue(h._running)
+        h.start()
+        h.stop()
+        h.stop()
+        self.assertFalse(h._running)
+
 
 class TestThreadingAsync(unittest.TestCase):
     def _makeOne(self, *args):
@@ -174,6 +146,31 @@ class TestThreadingAsync(unittest.TestCase):
         cv.wait()
         eq_(lst, ['oops'])
 
+    def test_wait(self):
+        mock_handler = mock.Mock()
+        async = self._makeOne(mock_handler)
+
+        lst = []
+        bv = threading.Event()
+        cv = threading.Event()
+
+        def wait_for_val():
+            bv.set()
+            try:
+                val = async.wait()
+            except ImportError:
+                lst.append('oops')
+            else:
+                lst.append(val)
+            cv.set()
+        th = threading.Thread(target=wait_for_val)
+        th.start()
+        bv.wait()
+
+        async.set("fred")
+        cv.wait()
+        eq_(lst, [True])
+
     def test_set_before_wait(self):
         mock_handler = mock.Mock()
         async = self._makeOne(mock_handler)
@@ -264,3 +261,46 @@ class TestThreadingAsync(unittest.TestCase):
         async.unlink(add_on)
         async.set('fred')
         assert not mock_handler.completion_queue.put.called
+
+
+class TestPeekableQueue(unittest.TestCase):
+    def _makeOne(self):
+        from kazoo.handlers.threading import _PeekableQueue
+        return _PeekableQueue()
+
+    def test_noblock(self):
+        from kazoo.handlers.threading import SequentialThreadingHandler
+        q = self._makeOne()
+
+        @raises(SequentialThreadingHandler.empty)
+        def testit():
+            return q.peek(block=False)
+        testit()
+
+    def test_peek_bad_timeout(self):
+        q = self._makeOne()
+
+        @raises(ValueError)
+        def testit():
+            return q.peek(timeout=-1)
+        testit()
+
+    def test_peek_value_add(self):
+        q = self._makeOne()
+        lst = []
+        cv = threading.Event()
+        bv = threading.Event()
+
+        def wait_for_val():
+            bv.set()
+            val = q.peek()
+            lst.append(val)
+            cv.set()
+        th = threading.Thread(target=wait_for_val)
+        th.start()
+        bv.wait()
+
+        eq_(lst, [])
+        q.put(1)
+        cv.wait()
+        eq_(lst, [1])
